<?php declare(strict_types=1);

namespace EwKit\Theme;

use EwKit\Theme\Interfaces\ThemeServiceStartup;
use EwKit\Theme\Services\ContentProviderServiceStartup;
use EwKit\Theme\Services\ImengineServiceStartup;
use EwKit\Theme\Services\MenuServiceStartup;
use EwKit\Theme\Services\NewsMLParserServiceStartup;
use EwKit\Theme\Services\ScriptsServiceStartup;
use EwKit\Theme\Services\SidebarServiceStartup;
use EwKit\Theme\Services\WidgetServiceStartup;
use EwKit\Theme\Services\TwigServiceStartup;
use Infomaker\Everyware\Base\AppConfig;
use Infomaker\Everyware\Base\Interfaces\ThemeStartup;
use Infomaker\Everyware\Base\OcPropertyMapConfig;
use Infomaker\Everyware\Base\PropertyMapConfig;
use Infomaker\Everyware\Support\Environment;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\ViewSetup;
use OpenContent;
use RuntimeException;

/**
 * Class Startup
 * @package EwKit\Theme
 */
class Startup implements ThemeStartup
{
    private $services;

    public function bootstrap(): void
    {
        $config = new OcPropertyMapConfig();

        try {
            $this->registerServices($config);
        } catch (RuntimeException $e) {
            NewRelicLog::error('Failed to register Services for theme');
        }

        $this
            ->setupServices()
            ->setupFilters()
            ->setupSearchPage('Search');
    }

    private function setupFilters(): Startup
    {
        \add_filter('show_admin_bar', '__return_false');
        \add_filter('request', [&$this, 'routeCustomUrls']);
        \add_filter('query_vars', [&$this, 'addCustomQueryVars']);

        return $this;
    }

    private function setupSearchPage(string $pageName): Startup
    {
        $pageSlug = Str::slug($pageName);

        // Convert WP-search to custom search for safety
        add_action('template_redirect', function () use ($pageSlug) {
            $query = $_GET;
            if (isset($query['s'])) {
                $query['q'] = $query['s'];
                unset($query['s']);
                wp_redirect(home_url("/{$pageSlug}/?") . http_build_query($query));
                exit();
            }
        });

        return $this;
    }

    private function routeArchiveUrls(array $queryVars = []): array
    {
        if (isset($queryVars['year'])) {
            $queryVars['post_type'] = 'article';
        }

        return $queryVars;
    }

    public function routeCustomUrls(array $queryVars = []): array
    {
        $queryVars = $this->routeArchiveUrls($queryVars);

        return $queryVars;
    }

    public function addCustomQueryVars(array $queryVars = []): array
    {
        return $queryVars;
    }

    private function setupServices(): Startup
    {
        foreach ($this->services as $service) {
            if ($service instanceof ThemeServiceStartup) {
                $service->setup();
            }
        }

        return $this;
    }

    private function registerServices(PropertyMapConfig $config): Startup
    {
        if ( ! class_exists('OpenContent')) {
            throw new RuntimeException('The OpenContent class does not seem to exist. Make sure the Everyware plugin is loaded correctly.');
        }

        $this->services = [
            new TwigServiceStartup(ViewSetup::getInstance()),
            new ContentProviderServiceStartup($config),
            new ImengineServiceStartup(OpenContent::getInstance()),
            new MenuServiceStartup(),
            new NewsMLParserServiceStartup(),
            new ScriptsServiceStartup(AppConfig::getOrganisationSlug(), ! Environment::isDev() ? GIT_COMMIT : false),
            new SidebarServiceStartup(),
            new WidgetServiceStartup()
        ];

        return $this;
    }
}
