# Webpack config

Webpack handles the compiling of all css/scss, js, image and font files. It also helps with notifying when certain css/js  coding rules are broken.


### Important: 
Webpack is a javascript bundler, therefor it can not natively handle other files than .js. We work around this by importing all css/images/fonts into javascript files and with the help of multiple plugins extracting those files from their javascript files into separate files in the assets folder. E.g by default we import style.scss in style.js. Webpack will then extract that scss file into assets/css/[name].css, [name] being the name of the javascript file the css file where imported into.


Webpack comes with the following paths pre defined. It'll compile all the files from these directories and output them into the assets folder.
If you wish to add a new file, customize or remove a existing one this is where you would do it. 

##### webpack/webpack.common.js
```JavaScript
entry: {
     article: uncompiledAssets + '/js/article/article.js',
     body: uncompiledAssets + '/js/body/body.js',
     head: uncompiledAssets + '/js/head.js',
     admin: uncompiledAssets + '/js/admin/admin.js', 
     style: uncompiledAssets + '/js/style.js'
}
```

For example if you organize your javascript files in the following structure

```
- js
  - article
      article.js
        - parts
            body.js
            header.js
            ...
  - body
      body.js
        - parts
            content.js
            ...
  ...
```
In the above example article.js and body.js would directly or indirectly be linked to all the files in their subdirectories, those files will then be bundled together and outputted in the following structure.
```
- assets
  - js
    article.min.js
    body.min.js
  ...
```
The compiled javascript assets gets outputted in assets/js folder declared in the output object. [name] is the path key in the entry object.

```JavaScript
output: {
    filename: devMode ? 'js/[name].js' : 'js/[name].min.js',
    path: compiledAssets,
}
```
The compiled and extracted css files get outputted in assets/css declared in MiniCssExtractPlugin. Important to notice that the css files inherit their names from the javascript file they are included in
```JavaScript
new MiniCssExtractPlugin({
    filename: devMode ? 'css/[name].css' : 'css/[name].min.css',
}),
```

### Stylelint & eslint

Whenever webpack compiles it will run the css and js files against certain rules declared in .eslintrc.js and .stylelintrc. You can easily customize these rules to whatever fit your needs or coding preferences. 

Learn more about how you customize your css rules [here](https://stylelint.io/) and your javascript rules [here.](https://eslint.org/docs/rules/)

If you wish to disable stylelint remove the following plugin from webpack.common.js
```JavaScript
new StyleLintPlugin({
    formatter: require('stylelint').formatters.string,
    syntax: 'scss',
    files: uncompiledAssets + '/scss/**',
    configFile: root + '/webpack/.stylelintrc'
})
```

And to remove eslint remove the following loader
```JavaScript
{
    enforce: "pre",
    test: /\.m?js$/,
    exclude: /node_modules/,
    loader: "eslint-loader", options: {
         failOnWArning: false,
         failOnError: false,
         configFile: root + '/webpack/.eslintrc.js'
    }
}
```

## Usage

Make sure you've installed all the necessary packages(npm install).

To run webpack you use any of the following commands:

```
npm run build
```

Is used before you push to production. More then compiling it also minify, optimize and extract sourcemaps. This is a slow process and only necessary to run before pushing to production.

```
npm run start
```

compiles all files and output them to the asset files without running all optimizations and minifications. This process is quicker and suitable for development.

```
npm run watch
```

Runs the same flow as npm run start except it is a continues process, it will watch all the files in the uncompiled folder for changes and automatically compile them for you.

```
npm lint:css
```

Fix some, but far from all css lint errors. It can handle simpler errors such as forgotten semicolons and inconsistent indentions.

```
npm lint:js
```

Same as lint:css but for javascript files.


