"use strict";
const path = require('path'),
    CleanWebpackPlugin = require('clean-webpack-plugin'),
    MiniCssExtractPlugin = require("mini-css-extract-plugin"),
    postcssPresetEnv = require('postcss-preset-env'),
    WebpackBuildNotifierPlugin = require('webpack-build-notifier'),
    StyleLintPlugin = require('stylelint-webpack-plugin');


/**
 * This is the configurations that are shared between prod and dev.
 */
//TODO Potentially add image-loader to fonts, logs, alias_resolve, globals
module.exports = env => {
    const devMode = env !== 'production',
        root = path.resolve(__dirname, '../'),
        uncompiledAssets = root + '/uncompiled-assets',
        compiledAssets = root + '/assets',
        pathsToClean = [
            compiledAssets,
        ],
        cleanOptions = {
            root: root,
            // exclude:  ['shared.js'], //If you wish to exclude any files from being erased add it in this array
            verbose: true,
            dry: false
        };


    return {
        //If you wish to change what files that gets outputted by webpack, add or remove them from this array.
        entry: {
            article: uncompiledAssets + '/js/article/article.js',
            body: uncompiledAssets + '/js/body/body.js',
            head: uncompiledAssets + '/js/head.js',
            admin: uncompiledAssets + '/js/admin/admin.js',
            style: uncompiledAssets + '/js/style.js',
        },
        //This is the the output path for compiled files.
        output: {
            filename: devMode ? 'js/[name].js' : 'js/[name].min.js',
            path: compiledAssets,
        },
        //Splitchunks helps us extract common dependencies into a shared seperate chunk
        optimization: {
            splitChunks: {
                chunks: 'all'
            }
        },
        /**
         * Loaders always load from reverse order. i.e from bottom to top.
         * 
         */
        module: {
            rules: [
                {
                    enforce: "pre",
                    test: /\.m?js$/,
                    exclude: /node_modules/,
                    loader: "eslint-loader", options: {
                        failOnWArning: false,
                        failOnError: false,
                        configFile: root + '/webpack/.eslintrc.js',
                        cache: true,
                        emitWarning: devMode ? true : false
                    }
                },
                {
                    test: /\.m?js$/,
                    exclude: /node_modules/,
                    use: {
                        //Transpile es6 to es5
                        loader: 'babel-loader', options: {
                            cacheDirectory: true,
                            presets: [
                                [
                                    "@babel/preset-env",
                                    {
                                      "useBuiltIns": "usage",
                                      "debug": false,
                                    }
                                ]
                            ],
                            plugins: [
                              "@babel/plugin-syntax-dynamic-import",
                              "@babel/plugin-proposal-class-properties",
                              "@babel/plugin-transform-arrow-functions",
                            ]
                        }
                    }
                },
                {
                    test: /\.(sa|sc|c)ss$/,
                    exclude: /node_modules/,
                    use: [
                        {
                            //In devmode we prefer not to extract the css, instead we add it to the html directly with a <style> tag.
                            //This improves the overall compile time.
                            //loader:  'style-loader'
                            loader: MiniCssExtractPlugin.loader,
                        }, 
                        {
                            //Loads all css found in the bundle and passes it to style-loader(dev) or minicss(prod)
                            loader: 'css-loader', options: {
                                sourceMap: true,
                                importLoaders: 1
                            }
                        }, 
                        {
                            //Postcss-loader handles transpiling css-syntax, linting etc.
                            loader: 'postcss-loader', options: {
                                sourceMap: true,
                                ident: 'postcss',
                                plugins: () => [
                                    postcssPresetEnv({
                                        autoprefixer: {
                                            grid: 'autoplace'
                                        }
                                    })
                                ]
                            }
                        }, 
                        {
                            //Compiles sass to css
                            loader: 'sass-loader', options: {
                                sourceMap: true
                            }
                        }
                    ],
                },
                {
                    test: /\.(png|svg|jpg|gif)$/,
                    use: [
                        {
                            //Resolves import/require on images and emits them into 'images' directory.
                            loader: 'file-loader', options: {//url-loader, vs file-leoader- vs copywebpackplugin???, set a limit
                                name: 'images/[name].[ext]',
                                publicPath: '../'
                            }
                        },
                        {
                            //Minify and optimize images
                            loader: 'image-webpack-loader',
                            options: {
                                disable: false,
                            },
                        },
                    ]
                },
                {
                    //Resolves import/require on fonts and emits them into 'fonts' directory.
                    test: /\.(woff|woff2|eot|ttf|otf)$/,
                    use: {
                        loader: 'file-loader', options: {
                            name: 'fonts/[name].[ext]'
                        }
                    }
                },
            ]
        },
        plugins: [
            //Completely cleans all files from the output folder before emitting the newly compiled files.
            new CleanWebpackPlugin(pathsToClean, cleanOptions),    
            //Provides a notification popup whenever the bundle compiles.
            new WebpackBuildNotifierPlugin({
                title: "Super awesome build",
                logo: path.resolve("./img/favicon.png"),
                sound: false,
                suppressSuccess: false
            }),
            new StyleLintPlugin({
                formatter: require('stylelint').formatters.string,
                content: uncompiledAssets + 'scss',
                syntax: 'scss',
                configFile: root + '/webpack/.stylelintrc',
                //cache: true,
                quiet: true,
            }),
            //Extracts the css from the javascript bundle and outputs it in css folder.
            new MiniCssExtractPlugin({
                filename: 'css/[name].min.css'
            })
        ]
    }
};